package com.elitesland.router.controller;


import com.elitesland.router.bean.GatewayRouteDefinition;
import com.elitesland.router.service.DynamicRouteService;
import com.elitesland.router.service.GatewayRoutesInNacosConfigService;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.config.PropertiesRouteDefinitionLocator;
import org.springframework.cloud.gateway.route.InMemoryRouteDefinitionRepository;
import org.springframework.cloud.gateway.route.Route;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import java.util.ArrayList;

/*
 *@Author Horbin.Zhang
 *@Date 2021-02-10 19:17
 *@Description
 * http://localhost:9040/actuator/gateway/routes 通过acutator查询route配置信息
 * 在生产环境中配置信息将通过redis进行查询及持久化操作
 */
@RestController
@RequestMapping("/route")
public class RouteController {
    @Autowired
    private GatewayRoutesInNacosConfigService gatewayRoutesInNacosConfigService;
    @Autowired
    PropertiesRouteDefinitionLocator PropertiesRouteDefinitionLocator;

    @Autowired
    private DynamicRouteService dynamicRouteService;


    /**
     *增加路由
     */
    @PostMapping("/addOrUpdate")
    public GatewayRouteDefinition addOrUpdate(@RequestBody GatewayRouteDefinition gwdefinition) {
        return gatewayRoutesInNacosConfigService.addOrUpdateRouteDefinitiToNacosServer(gwdefinition);
    }

    /**
     *删除路由
     */
    @DeleteMapping("/routes/{id}")
    public String delete(@PathVariable String id) {
        return gatewayRoutesInNacosConfigService.deleteRouteDefinitiToNacosServer(id);
    }

    /**
     * 获取nacos里配置的动态路由配置信息
     * */
    @GetMapping("/routes/all")
    public Flux<ArrayList<GatewayRouteDefinition>> all() {
        Gson g = new Gson();
        ArrayList<GatewayRouteDefinition> list = new ArrayList<>();
        JsonArray gatewayDefinitionsFromNacosServer = gatewayRoutesInNacosConfigService.getGatewayDefinitionsFromNacosServer();
        gatewayDefinitionsFromNacosServer.forEach(o ->{
            GatewayRouteDefinition gatewayRouteDefinition = g.fromJson(o, GatewayRouteDefinition.class);
            list.add(gatewayRouteDefinition);
        });
        return Flux.just(list);
    }
    /**
     * 获取本地所有的动态路由配置信息
     * */
    @GetMapping("/all")
    public Flux<RouteDefinition> all1() {
        Flux<RouteDefinition> routeDefinitions = PropertiesRouteDefinitionLocator.getRouteDefinitions();
        routeDefinitions.subscribe();
        return routeDefinitions;
    }
}